use crate::admin::domain::{AdminApp, AdminModel};
use axum::routing::MethodRouter;
use axum::{async_trait, response::IntoResponse};
use axum::{routing::get, Router};
use core::future::Future;
use std::collections::HashMap;
use std::sync::Arc;

pub type AdminState = Arc<AdminRegistry>;

// main registry.
#[derive(Clone)]
pub struct AdminRegistry {
    base_path: String,
    apps: HashMap<String, internal::DataNode>,
    models: HashMap<String, internal::DataNode>,
}

impl AdminRegistry {
    pub fn new(base_path: &str) -> Self {
        AdminRegistry {
            base_path: base_path.to_owned(),
            apps: HashMap::new(),
            models: HashMap::new(),
        }
    }

    pub fn generate_router<T>(&self) -> Router<T>
    where
        T: Clone,
        T: Send,
        T: Sync,
        T: 'static,
        crate::service::templates::Templates: axum::extract::FromRef<T>,
        AdminState: axum::extract::FromRef<T>,
    {
        routing::generate_routes::<T>(&self)
    }

    pub fn get_apps(&self) -> Vec<AdminApp> {
        self.apps
            .iter()
            .map(|(key, node)| self.get_app(key, node))
            .collect()
    }

    fn get_app(&self, name: &str, node: &internal::DataNode) -> AdminApp {
        let label = node.label.as_ref().unwrap_or(&String::new()).clone();
        AdminApp {
            name: name.to_owned(),
            app_label: label,
            app_url: "".to_owned(),
            models: vec![],
        }
    }

    pub fn register_app(&mut self, name: &str, app_label: &str, app_url: &str) {
        self.apps.insert(
            name.to_owned(),
            internal::DataNode {
                object_name: name.to_owned(),
                label: Some(app_label.to_owned()),
            },
        );
    }

    pub fn get_models(&self) -> Vec<AdminModel> {
        vec![]
    }

    pub fn get_models_for_app(&self, name: &str) -> Vec<AdminModel> {
        vec![]
    }

    pub fn register_model(&mut self, name: &str, config: AdminModelConfig) {}
}

// user uses this configuration object to register another model.
pub struct AdminModelConfig {
    pub app: Arc<str>,
    pub custom_index_handler: Option<MethodRouter>,
}

mod internal {
    #[derive(Clone)]
    pub struct DataNode {
        pub object_name: String,
        pub label: Option<String>,
    }
}

mod routing {
    use axum::{routing::get, Router};

    pub fn generate_routes<T>(registry: &super::AdminRegistry) -> Router<T>
    where
        T: Clone,
        T: Send,
        T: Sync,
        T: 'static,
        crate::service::templates::Templates: axum::extract::FromRef<T>,
        super::AdminState: axum::extract::FromRef<T>,
    {
        let mut router: Router<T> = Router::new();
        let apps = registry.get_apps();
        router = router.route(
            &format!("/{}", registry.base_path),
            get(crate::admin::views::index),
        );
        for app in apps {
            let app_route = format!("/{}/{}", registry.base_path, app.app_url);

            // Add a route for the app
            router = router.route(
                &app_route,
                get(move || async move { format!("Admin panel for {}", app.app_label) }),
            );

            // Add routes for each model in the app
            /*for model in data_node.models.iter() {
                let model_route = format!("{}/{}", app_route, model.name);
                router = router.route(
                    &model_route,
                    get(move || async move { format!("Model page for {}", model.name) }),
                );
            }*/
        }

        router
    }

    fn define_example_data(registry: &mut super::AdminRegistry) {
        /*let models = vec![
            AdminModel {
                name: "User".to_owned(),
                object_name: "User".to_owned(),
                admin_url: "/admin/users/user".to_owned(),
                view_only: false,
                add_url: Some("/admin/users/user/add".to_owned()),
            },
            AdminModel {
                name: "Group".to_owned(),
                object_name: "Group".to_owned(),
                admin_url: "/admin/users/group".to_owned(),
                view_only: false,
                add_url: None,
            },
            AdminModel {
                name: "Permission".to_owned(),
                object_name: "Permission".to_owned(),
                admin_url: "/admin/users/permission".to_owned(),
                view_only: true,
                add_url: None,
            },
        ];*/
        registry.register_app("auth", "Authorities", "auth");
    }
}
