use std::fs;
use std::path::{Path, PathBuf};

#[derive(Debug)]
pub struct PathEntry {
    path: PathBuf,
    file_name: String,
    is_dir: bool,
    is_file: bool,
    is_symlink: bool,
}

impl std::fmt::Display for PathEntry {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(f, "{}", self.path.to_string_lossy())
    }
}

struct FolderScanner {
    base_path: PathBuf,
}

impl FolderScanner {
    // Create a new FolderScanner with a base path
    fn new(base_path: &str) -> Self {
        FolderScanner {
            base_path: PathBuf::from(base_path),
        }
    }

    // Scan the folder with optional filters
    fn scan_folder(&self, folder: &str, filters: Option<Filters>) -> Vec<PathEntry> {
        let base_path_clean = dunce::canonicalize(&self.base_path).unwrap();
        let full_path = dunce::canonicalize(self.base_path.join(folder)).unwrap();

        println!("Scanning folder: {}", &full_path.display());

        // Ensure that the path is within the base path
        if !full_path.starts_with(&base_path_clean) {
            panic!("Access to the folder outside the base path is not allowed.");
        }

        let entries = fs::read_dir(full_path).unwrap();
        let mut files = Vec::new();

        for entry in entries {
            if let Ok(dir_entry) = entry {
                let path = dir_entry.path();

                if let Some(f) = &filters {
                    if f.should_filter(&path) {
                        continue;
                    }
                }

                let (is_dir, is_file, is_simlink) = if let Ok(metadata) = dir_entry.metadata() {
                    (metadata.is_dir(), metadata.is_file(), metadata.is_symlink())
                } else {
                    (false, false, false)
                };

                files.push(PathEntry {
                    path: path,
                    file_name: dir_entry.file_name().to_string_lossy().into_owned(),
                    is_dir: is_dir,
                    is_file: is_file,
                    is_symlink: is_simlink,
                });
            }
        }

        files
    }
}

struct Filters {
    exclude_dot_files: bool,
    exclude_folders: bool,
    exclude_symlinks: bool,
    allowed_extensions: Vec<String>,
}

impl Filters {
    fn should_filter(&self, path: &Path) -> bool {
        if self.exclude_dot_files && path.file_name().unwrap().to_str().unwrap().starts_with('.') {
            return true;
        }

        if self.exclude_folders && path.is_dir() {
            return true;
        }

        if self.exclude_symlinks && fs::symlink_metadata(path).unwrap().file_type().is_symlink() {
            return true;
        }

        if !self.allowed_extensions.is_empty() {
            if let Some(ext) = path.extension() {
                if !self
                    .allowed_extensions
                    .contains(&ext.to_str().unwrap().to_string())
                {
                    return true;
                }
            } else {
                return true;
            }
        }

        false
    }
}

fn main() {
    let scanner = FolderScanner::new(".");

    // Example usage
    let filters = Filters {
        exclude_dot_files: false,
        exclude_folders: false,
        exclude_symlinks: false,
        allowed_extensions: vec!["md".to_string(), "rs".to_string()],
    };

    let files = scanner.scan_folder("src", Some(filters));
    for file in files {
        println!("{}", file); // Display
        println!("{:?}", file); // Debug
    }
}
