/* Default handlers */
use crate::service::templates::Templates;
use axum::{
    body::{boxed, Full},
    extract::State,
    http::{header, StatusCode, Uri},
    response::{IntoResponse, Response},
};
use rust_embed::RustEmbed;
use std::marker::PhantomData;

// usage: .route_service("/static/*file", static_handler.into_service())
pub async fn static_handler(uri: Uri) -> impl IntoResponse {
    let mut path = uri.path().trim_start_matches('/').to_string();

    if path.starts_with("static/") {
        path = path.replace("static/", "");
    }

    StaticFile::get(path)
}

// usage: .fallback(not_found_handler)
pub async fn not_found_handler(
    State(templates): State<Templates>,
) -> axum::response::Result<impl IntoResponse> {
    not_found(templates)
}

fn not_found(templates: Templates) -> axum::response::Result<impl IntoResponse> {
    let body = templates.render_html("http404.html", ());
    Ok((StatusCode::NOT_FOUND, body))
}

pub struct EmbeddedFile<E, T> {
    pub path: T,
    embed: PhantomData<E>,
}

impl<E, T> EmbeddedFile<E, T> {
    pub fn get(path: T) -> Self {
        Self {
            path,
            embed: PhantomData,
        }
    }
}

impl<E, T> IntoResponse for EmbeddedFile<E, T>
where
    E: RustEmbed,
    T: AsRef<str>,
{
    fn into_response(self) -> Response {
        let path: &str = self.path.as_ref();
        match E::get(path) {
            Some(content) => {
                let body = boxed(Full::from(content.data));
                let mime = mime_guess::from_path(path).first_or_octet_stream();
                Response::builder()
                    .header(header::CONTENT_TYPE, mime.as_ref())
                    .body(body)
                    .unwrap()
            }
            None => StatusCode::NOT_FOUND.into_response(),
        }
    }
}

#[derive(RustEmbed)]
#[folder = "static"]
struct StaticDir;
type StaticFile<T> = EmbeddedFile<StaticDir, T>;
